﻿/*
 * XPT2046_TP_Correct_Pos.c
 *
 *  Created on: 2024/11/18
 *      Author: egaono-electronics
 */

#include "XPT2046_TP_Correct_Pos.h"

#include "XPT2046_TP_Access.h"



// Y位置補正係数
float GLB_flt_Tilt_TP_Y = 0;    // 水平方向のゆがみ率
float GLB_flt_Tilt_V_TP_Y = 0;  // 垂直方向のゆがみ率



//----------------------------------------------------------
// タッチパネルのY位置補正係数算出
//----------------------------------------------------------
// 引数;
// struct sParam_Adjust_TP *pstructParam_Adjust_TP :
//          タッチパネル位置調整用パラメータ構造体のポインタ


// 戻り値;
//    なし
//----------------------------------------------------------
void Calc_Correction_Coefficent_Pos_TP_Y(
            struct sParam_Adjust_TP *pstructParam_Adjust_TP)
{
  //------------------------------------------------------------
  // X側補正係数 : 水平方向のゆがみ率を算出
  //------------------------------------------------------------
  GLB_flt_Tilt_TP_Y
    = (float)(pstructParam_Adjust_TP->int16_Adjust_TP_Y_2
    - pstructParam_Adjust_TP->int16_Adjust_TP_Y_1)
    / (float)(pstructParam_Adjust_TP->int16_Adjust_TP_X_MAX
    - pstructParam_Adjust_TP->int16_Adjust_TP_X_MIN);
  //------------------------------------------------------------

  //------------------------------------------------------------
  // Y側補正係数 : 垂直方向のゆがみ率を算出
  //------------------------------------------------------------
  GLB_flt_Tilt_V_TP_Y
        = (float)(pstructParam_Adjust_TP->int16_Adjust_TP_Y_3
        - pstructParam_Adjust_TP->int16_Adjust_TP_Y_1)
        / (float)(pstructParam_Adjust_TP->int16_Adjust_TP_Y_4
        - pstructParam_Adjust_TP->int16_Adjust_TP_Y_2);
  //------------------------------------------------------------
}



//----------------------------------------------------------
// タッチパネルのY位置補正値算出
//----------------------------------------------------------
// 引数;
// struct sParam_Adjust_TP *pstructParam_Adjust_TP :
//          タッチパネル位置調整用パラメータ構造体のポインタ


// 戻り値:
//  Y位置補正値
//----------------------------------------------------------
uint16_t Correct_Pos_TP_Y(
            struct sParam_Adjust_TP *pstructParam_Adjust_TP)
{
#ifdef def_Correct_Pos_TP_Y
  //----------------------------------------
  // 補正あり
  //----------------------------------------
  float flt_Work;
  uint16_t uint16_Y_rev;
  uint16_t uint16_Correct_Value;


  //------------------------------------------------------------
  // Y位置補正
  //------------------------------------------------------------
  // 水平方向の補正値を算出
  uint16_Y_rev
        = (uint16_t)((float)(GLB_structParam_Position_TP.
                                        uint16_Position_TP_X
        - pstructParam_Adjust_TP->int16_Adjust_TP_X_MIN)
        * GLB_flt_Tilt_TP_Y);

  // 水平方向の補正値を引く
  flt_Work = (float)(GLB_structParam_Position_TP.
                        uint16_Position_TP_Y - uint16_Y_rev);

  // 垂直方向のゆがみ率を乗算して垂直方向の補正を行う。
  uint16_Correct_Value
                = (uint16_t)(flt_Work * GLB_flt_Tilt_V_TP_Y);
  //------------------------------------------------------------


  return(uint16_Correct_Value);
#else  // def_Correct_Pos_TP_Y
  //----------------------------------------
  // 補正なし
  //----------------------------------------
  uint16_t uint16_Correct_Value;


  uint16_Correct_Value = GLB_uint16_Position_TP_Y;


  return(uint16_Correct_Value);
#endif  // def_Correct_Pos_TP_Y
}
