﻿/*
 * MSP2807_Test_TP_Key_In.c
 *
 *  Created on: 2025/02/25
 *      Author: egaono-electronics
 */

#include "MSP2807_Test_TP_Key_In.h"

#include "ILI9341_Display_Clear.h"
#include "ILI9341_Display_ASCII.h"

#include "MSP2807_Test_TP_Detect.h"


#include <string.h>



//--------------------------------------------------------------
// Test TP SWデータ入力
//--------------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// struct sParam_Position_TP *pstructParam_Position_TP :
//          タッチパネル位置取得用パラメータの構造体のポインタ

// struct sParam_TP_SW *pstructParam_TP_SW :
//          タッチパネルスイッチ用パラメータの構造体のポインタ

// uint16_t uint16_Offset_Draw_Y : 表示位置オフセット Y

// int16_t int16_Size_Data_Buffer : Data Buffer Size

// int16_t *pint16_Size_Input_Data :
//                  入力データサイズを格納する変数のポインタ
// uint8_t *puint8_aData_Buffer : Dataを格納するBufferのポインタ


// 戻り値：
//   -2 : Block, Sequence, Type 変更
//   -1 : 処理中
//    0 : OK終了 : Enter
//    1 : NG終了
//--------------------------------------------------------------
int16_t Input_Data_Test_TP_SW(uint8_t *puint8_JobNum,
            struct sParam_Position_TP *pstructParam_Position_TP,
            struct sParam_TP_SW *pstructParam_TP_SW,
                              uint16_t uint16_Offset_Draw_Y,
                              int16_t int16_Size_Data_Buffer,
                              int16_t *pint16_Size_Input_Data,
                              uint8_t *puint8_aData_Buffer)
{
  static uint8_t STC_uint8_JobNum_TP;  // TP処理番号


  static int16_t STC_int16_Count;
  static uint16_t STC_uint16_Position;

  static int16_t STC_int16_Invert;

  int16_t int16_Key_In_Type;
  uint8_t uint8_Key_In_Data;

  int16_t int16_I;

  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    STC_uint8_JobNum_TP = 0;  // TP処理番号初期化

    GLB_int16_Count_TP_Key_In = 0;
    GLB_uint16_Position_TP_Key_In = 0;


    // Key In Buffer初期化
    for(int16_I = 0; int16_I < int16_Size_Data_Buffer; int16_I++){
      GLB_int16_aTP_Key_In_Buffer[int16_I] = -1;
    }


    if(*pint16_Size_Input_Data > 0){
      // 入力データ数が 1 以上の場合、
      //        データをセットして表示処理に移行する。
      GLB_int16_Count_TP_Key_In = *pint16_Size_Input_Data;

      for(int16_I = 0;
            int16_I < GLB_int16_Count_TP_Key_In; int16_I++){
        GLB_int16_aTP_Key_In_Buffer[int16_I]
              = (int16_t)((uint16_t)puint8_aData_Buffer[int16_I]);
      }

      GLB_uint16_Position_TP_Key_In = GLB_int16_Count_TP_Key_In;
      if(GLB_int16_Count_TP_Key_In == int16_Size_Data_Buffer){
        GLB_uint16_Position_TP_Key_In--;
      }

      STC_int16_Count = int16_Size_Data_Buffer;

      STC_uint16_Position = 0;
      STC_int16_Invert = 0;


      *puint8_JobNum = 2;  // データ表示処理に移行する。: case 2:
      int16_Return = -1;   // 処理継続
      break;
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。: case 1:
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //--------------------------------------------------------------
    // タッチパネルKeyデータ入力
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Position_TP *pstructParam_Position_TP :
    //          タッチパネル位置取得用パラメータの構造体のポインタ

    // struct sParam_TP_SW *pstructParam_TP_SW :
    //          タッチパネルスイッチ用パラメータの構造体のポインタ

    // int16_t int16_Sel : 動作選択
    //    0 : Alpha Numeric
    //    1 : HEX_In

    // int16_t int16_Max_Num_SW : スイッチ番号最大値


    // 戻り値：
    //    -2 : NG終了
    //    -1 : 処理中
    // 0以上 : OK : Switch番号 or HEXデータ
    //--------------------------------------------------------------
    int16_Return = TP_Key_In_Data(&STC_uint8_JobNum_TP,
                                  pstructParam_Position_TP,
                                  pstructParam_TP_SW,
                                  GLB_int16_Flag_AN_HEX,
                                  (int16_t)def_Num_TP_SW_Key_Func);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    if(int16_Return < -1){
      // NG
      int16_Return = -1;  // 処理中をセット
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    //--------------------------------------------------------------
    // Switch番号 or HEXデータ の判定
    //--------------------------------------------------------------
    if(int16_Return <= (int16_t)def_Num_TP_SW_Al_Nu){
      // Alpha Numeric Key : LF以下の番号
      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_Data;

      // Alpha Numericデータ番号セット
      uint8_Key_In_Data = GLB_uint8_aTP_Key_In_Char[int16_Return];
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_BK){
      // BK : Back
      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_BK;
      uint8_Key_In_Data = 0;
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_FW){
      // FW : Forward
      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_FW;
      uint8_Key_In_Data = 0;
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_BS){
      // BS : Back Space
      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_BS;
      uint8_Key_In_Data = 0;
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_DL){
      // DL : delete
      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_DL;
      uint8_Key_In_Data = 0;
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_ET){
      // ET : Enter

      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_ET;
      uint8_Key_In_Data = 0;


      *pint16_Size_Input_Data = GLB_int16_Count_TP_Key_In;

      // 入力データセット
      for(int16_I = 0;
            int16_I < GLB_int16_Count_TP_Key_In; int16_I++){
        puint8_aData_Buffer[int16_I]
              = (uint8_t)GLB_int16_aTP_Key_In_Buffer[int16_I];
      }

      //---------------------------------------
      // 処理終了
      //---------------------------------------
      *puint8_JobNum = 0;  // 処理番号初期化
      int16_Return = 0;    // OK終了
      break;
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_AH){
      // ASCII/HEX

      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_AH;
      uint8_Key_In_Data = 0;

      // Alpha Numeric/HEX切替
      if(GLB_int16_Flag_AN_HEX == 0){
        GLB_int16_Flag_AN_HEX = 1;  // HEX
      }
      else{
        GLB_int16_Flag_AN_HEX = 0;  // Alpha Numeric
      }


      //--------------------------------------------------------
      // TP SW Alpha Numericの値を設定
      //--------------------------------------------------------
      // 引数:
      // int16_t int16_A_H : ASCII/HEX選択
      //    0 : ASCII
      //    1 : HEX

      // int16_t int16_U_L :
      //        Upper case/Lower case 選択 : 大文字/小文字 選択
      //    0 : Upper case : 大文字
      //    1 : Lower case : 小文字

      // struct sParam_TP_SW *pstructParam_TP_SW :
      //        タッチパネルスイッチ用パラメータの構造体のポインタ


      // 戻り値：
      //    なし
      //-------------------------------------------------------
      Set_TP_SW_Value_Alpha_Numeric(GLB_int16_Flag_AN_HEX,
                                    GLB_int16_Flag_U_L,
                                    pstructParam_TP_SW);


      STC_int16_Count = int16_Size_Data_Buffer;

      STC_uint16_Position = 0;


      if(STC_uint16_Position == GLB_uint16_Position_TP_Key_In){
        STC_int16_Invert = 1;  // 反転表示
      }
      else{
        STC_int16_Invert = 0;  // 通常表示
      }


      *puint8_JobNum = 3;  // ASC/HEX表示 : case 3: に移行する。
      int16_Return = -1;
      break;
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_CL){
      // CL : Clear
      int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_CL;
      uint8_Key_In_Data = 0;
    }
    else if(int16_Return == (int16_t)def_Num_TP_SW_Func_UL){
      // UL : Upper case/Lower case

      // Upper case/Lower case切替
      if(GLB_int16_Flag_U_L == 0){
        GLB_int16_Flag_U_L = 1;  // Lower case : 小文字
      }
      else{
        GLB_int16_Flag_U_L = 0;  // Upper case : 大文字
      }


      //--------------------------------------------------------
      // TP SW Alpha Numericの値を設定
      //--------------------------------------------------------
      // 引数:
      // int16_t int16_A_H : ASCII/HEX選択
      //    0 : ASCII
      //    1 : HEX

      // int16_t int16_U_L :
      //        Upper case/Lower case 選択 : 大文字/小文字 選択
      //    0 : Upper case : 大文字
      //    1 : Lower case : 小文字

      // struct sParam_TP_SW *pstructParam_TP_SW :
      //    タッチパネルスイッチ用パラメータの構造体のポインタ


      // 戻り値：
      //    なし
      //--------------------------------------------------------
      Set_TP_SW_Value_Alpha_Numeric(GLB_int16_Flag_AN_HEX,
                                    GLB_int16_Flag_U_L,
                                    pstructParam_TP_SW);


      *puint8_JobNum = 4;  // タッチパネルのSW表示処理に移行する。
                           // : case 4:
      int16_Return = -1;   // 処理中をセット
      break;  // 処理継続
    }
    else{
      // HEXデータ入力時のスイッチ番号 : bit 8 が 1
      if(GLB_int16_Flag_AN_HEX == 1){
        int16_Key_In_Type = (int16_t)def_Type_TP_Key_In_Data;

        // HEXデータセット
        uint8_Key_In_Data = int16_Return & 0xFEFF;
      }
      else{
        // NG : HEXデータ入力時以外のときに、
        //          Alpha NumericとFunction以外のデータがきた場合
        break;
      }
    }
    //--------------------------------------------------------------



    //--------------------------------------------------------------
    // TP Key入力データをBufferにセットする。
    //--------------------------------------------------------------
    // 引数：
    // int16_t int16_Type : 動作選択
    //  -1 : BK : 戻る
    //   0 : Key In : Keyデータ入力
    //   1 : FW : 進む

    // uint8_t uint8_Data : Key入力データ
    // uint16_t *puint16_Position : 入力データの位置変数のポインタ
    // int16_t *pint16_Buffer :
    //                  入力データを格納するバッファのポインタ
    // int16_t *pint16_Count : データ数変数のポインタ
    // int16_t int16_Size_Buffer : Bufferのサイズ


    // 戻り値：
    //   -1 : Buffer Empty
    //    0 : OK
    //    1 : Buffer Full
    //    2 : End Position
    //    3 : NG
    //--------------------------------------------------------------
    int16_Return = Put_TP_Key_In_Data(int16_Key_In_Type,
                                      uint8_Key_In_Data,
                                      &GLB_uint16_Position_TP_Key_In,
                                      GLB_int16_aTP_Key_In_Buffer,
                                      &GLB_int16_Count_TP_Key_In,
                                      int16_Size_Data_Buffer);
    if(int16_Return != 0){
      // OK終了でない場合
      int16_Return = -1;  // 処理中をセット
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    STC_int16_Count = int16_Size_Data_Buffer;

    STC_uint16_Position = 0;


    if(STC_uint16_Position == GLB_uint16_Position_TP_Key_In){
      STC_int16_Invert = 1;  // 反転表示
    }
    else{
      STC_int16_Invert = 0;  // 通常表示
    }


    (*puint8_JobNum)++;  // 次の動作に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    // Keyデータ表示
    //--------------------------------------------------------------
    // TP Key入力データ表示
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納された変数のポインタ

    // int16_t int16_Type : 表示選択
    //  0 : ASCII
    //  1 : HEX

    // int16_t int16_Invert : 反転表示選択
    //  0 : 通常表示
    //  1 : 反転表示

    // uint16_t uint16_Offset_Draw_Y : 表示位置オフセット Y

    // int16_t *pint16_aData :
    //              入力データが格納されているバッファのポインタ
    // uint16_t uint16_Position : 入力データの位置


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = Draw_TP_Key_In_Data(&STC_uint8_JobNum_TP,
                                       GLB_int16_Flag_AN_HEX,
                                       STC_int16_Invert,
                                       uint16_Offset_Draw_Y,
                                       GLB_int16_aTP_Key_In_Buffer,
                                       STC_uint16_Position);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    STC_uint16_Position++;
    STC_int16_Count--;

    if(STC_uint16_Position == GLB_uint16_Position_TP_Key_In){
      STC_int16_Invert = 1;  // 反転表示
    }
    else{
      STC_int16_Invert = 0;  // 通常表示
    }

    // 終了判定
    if(STC_int16_Count > 0){
      int16_Return = -1;
      break;
    }


    (*puint8_JobNum)--;  // 処理番号デクリメント
                         // : ひとつ前の処理番号移行する。: case 1:
    int16_Return = -1;   // 処理継続
    break;

  case 3:
    //--------------------------------------------------------------
    // ASC/HEX表示
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納された変数のポインタ

    // int16_t int16_Sel : 表示選択
    //   -1 : ASCII : ASC without UL
    //    0 : ASCII : ASC
    //    1 : HEX : HEX 1
    //    2 : HEX : HEX 2

    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return
        = Draw_Msg_ASC_HEX(&STC_uint8_JobNum_TP,
                           GLB_int16_Flag_AN_HEX);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    (*puint8_JobNum)--;  // データ表示処理 : case 2: に移行する。
    int16_Return = -1;
    break;

  case 4:
    //--------------------------------------------------------------
    // タッチパネルのスイッチ表示 :
    //    開始スイッチ番号から終了スイッチ番号までを表示する。
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納された変数のポインタ

    // int16_t int16_Num_SW_Start : 開始スイッチ番号
    // int16_t int16_Num_SW_End : 終了スイッチ番号

    // uint8_t uint8_R : 反転表示フラグ
    //    0 : 通常表示
    //    1 : 反転表示

    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = Print_TP_SW_Num(&STC_uint8_JobNum_TP,
                                   (int16_t)def_Num_TP_SW_Key_A,
                                   (int16_t)def_Num_TP_SW_Al_Nu,
                                   (uint8_t)0);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    *puint8_JobNum = 1;  // タッチパネルKeyデータ入力処理に移行する。
                         //                 : case 1:
    int16_Return = -1;   // 処理継続
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}



//--------------------------------------------------------------
// Test タッチパネル上のアルファニューメリックデータ入力
//--------------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// struct sParam_Position_TP *pstructParam_Position_TP :
//          タッチパネル位置取得用パラメータの構造体のポインタ

// struct sParam_Adjust_TP *pstructParam_Adjust_TP :
//              タッチパネル位置調整用パラメータ構造体のポインタ

// struct sParam_TP_SW *pstructParam_TP_SW :
//              タッチパネルスイッチ用パラメータの構造体のポインタ

// uint16_t *puint16_Num_Data_Input :
//                      入力データ数を格納するWorkのポインタ
// uint8_t *puint8_aData : Dataを格納するBufferのポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//--------------------------------------------------------------
int16_t Input_Data_AN_Test_TP(uint8_t *puint8_JobNum,
                struct sParam_Position_TP *pstructParam_Position_TP,
                struct sParam_Adjust_TP *pstructParam_Adjust_TP,
                struct sParam_TP_SW *pstructParam_TP_SW,
                uint16_t *puint16_Num_Data_Input,
                uint8_t *puint8_aData)
{
  static uint8_t STC_uint8_JobNum_LCD;  // LCD処理番号


  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    STC_uint8_JobNum_LCD = 0;  // LCD処理番号初期化


    //--------------------------------------------------------------
    // Test TP SW パラメータセット
    //--------------------------------------------------------------
    //--------------------------------------------------------------
    // Test TP SW Alpha Numericの値を設定
    //--------------------------------------------------------------
    // 引数:
    // int16_t int16_A_H : ASCII/HEX選択
    //    0 : ASCII
    //    1 : HEX

    // int16_t int16_U_L :
    //          Upper case/Lower case 選択 : 大文字/小文字 選択
    //    0 : Upper case : 大文字
    //    1 : Lower case : 小文字

    // struct sParam_TP_SW *pstructParam_TP_SW :
    //          タッチパネルスイッチ用パラメータの構造体のポインタ


    // 戻り値：
    //    なし
    //--------------------------------------------------------------
    Set_TP_SW_Value_Alpha_Numeric(GLB_int16_Flag_AN_HEX,
                                  GLB_int16_Flag_U_L,
                                  pstructParam_TP_SW);


    //--------------------------------------------------------------
    // Test TP SW Functionの値を設定
    //--------------------------------------------------------------
    // 引数:
    // struct sParam_TP_SW *pstructParam_TP_SW :
    //          タッチパネルスイッチ用パラメータの構造体のポインタ


    // 戻り値：
    //    なし
    //--------------------------------------------------------------
    Set_Test_TP_SW_Value_Function(pstructParam_TP_SW);


    //--------------------------------------------------------------
    // TP SW Upper case/Lower caseの値を設定 : UL
    //--------------------------------------------------------------
    // 引数:
    // int16_t int16_Sel : 表示選択
    //    0 : 表示 : ASCII
    //    1 : 消去 : HEX

    // struct sParam_TP_SW *pstructParam_TP_SW :
    //          タッチパネルスイッチ用パラメータの構造体のポインタ


    // 戻り値：
    //    なし
    //--------------------------------------------------------------
    Set_TP_SW_Value_Upper_Lower(GLB_int16_Flag_U_L,
                                pstructParam_TP_SW);


    //--------------------------------------------------------------
    // Test TP SWの表示位置と検出位置をセット
    //--------------------------------------------------------------
    // struct sParam_Adjust_TP *pstructParam_Adjust_TP :
    //              タッチパネル位置調整用パラメータ構造体のポインタ

    // struct sParam_TP_SW *pstructParam_TP_SW :
    //              タッチパネルスイッチ用パラメータの構造体のポインタ


    // 戻り値：
    //    なし
    //--------------------------------------------------------------
    Set_Test_TP_SW_Position(pstructParam_Adjust_TP,
                            pstructParam_TP_SW);
    //--------------------------------------------------------------


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //--------------------------------------------------------------
    // Test データ入力画面表示
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = Print_Screen_Test_Input_Data(&STC_uint8_JobNum_LCD);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //--------------------------------------------------------------
    // TP SWデータ入力
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // struct sParam_Position_TP *pstructParam_Position_TP :
    //          タッチパネル位置取得用パラメータの構造体のポインタ

    // struct sParam_TP_SW *pstructParam_TP_SW :
    //          タッチパネルスイッチ用パラメータの構造体のポインタ

    // uint16_t uint16_Offset_Draw_Y : 表示位置オフセット Y

    // int16_t int16_Size_Data_Buffer : Data Buffer Size

    // int16_t *pint16_Size_Input_Data
    //                 : 入力データサイズを格納する変数のポインタ
    // uint8_t *puint8_aData_Buffer : Dataを格納するBufferのポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = Input_Data_Test_TP_SW(&STC_uint8_JobNum_LCD,
                            pstructParam_Position_TP,
                            pstructParam_TP_SW,
                            (uint16_t)def_Offset_Draw_Input_Data_Y,
                            (int16_t)def_Size_TP_Key_Data_Buffer,
                            (int16_t *)puint16_Num_Data_Input,
                            puint8_aData);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    *puint8_JobNum = 0;  // 処理番号初期化
    // int16_Return : 結果
    break;  // OK

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}



//--------------------------------------------------------------
// Test データ入力画面表示
//--------------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//--------------------------------------------------------------
int16_t Print_Screen_Test_Input_Data(uint8_t *puint8_JobNum)
{
  static uint8_t STC_uint8_JobNum_LCD;  // LCD処理番号

  static int16_t STC_int16_Num_SW_Start;
  static int16_t STC_int16_Num_SW_End;


  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    STC_uint8_JobNum_LCD = 0;  // LCD処理番号初期化


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //--------------------------------------------------------------
    // 画面表示をクリアする。
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK
    //    1 : NG
    //--------------------------------------------------------------
    int16_Return = Clear_Display(&STC_uint8_JobNum_LCD);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //-----------------------------------------------------------
    // Data入力 Label表示
    //-----------------------------------------------------------
    //--------------------------------------------------
    // 横長表示 右
    //--------------------------------------------------
    //--------------------------------------------------------------
    // LCD文字表示 : ASCII : Font選択 : ILI9341
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // int16_t int16_Check_Semaphore : セマフォチェック有無
    //    0 : チェックなし
    //    1 : チェックあり

    // uint16_t uint16_Font : font指定
    //    0 : 8 x 8 font : ASCII
    //    1 : 16 x 16 font : ASCII
    //    2 : 32 x 32 font : ASCII
    //    3 : 48 x 48 font : ASCII

    // uint8_t uint8_Oblong : 縦長/横長
    //    0 : Lengthwise : 縦長 左
    //    1 : Lengthwise : 縦長 右
    //    2 : WidthwiseL : 横長 左
    //    3 : WidthwiseR : 横長 右

    // uint8_t uint8_Type : 表示選択
    //      0 : 通常表示
    //      1 : 反転表示
    //      2 : 通常表示の上書き
    //      3 : 反転表示の上書き

    // uint32_t uint32_ColorCode : カラーコード

    // uint16_t uint16_StartX : 表示開始位置X
    // uint16_t uint16_StartY : 表示開始位置Y

    // uint16_t uint16_Length : 表示文字数
    // uint8_t *puint8_aData :
    //          表示文字コードが格納されているBufferのポインタ


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = ILI9341_SPI_Display_ASCII(&STC_uint8_JobNum_LCD,
                    1,
                    (uint16_t)def_Font_ASCII_16x16,
                    (uint8_t)def_Dir_Disp_LCD_Sel, 0,
                    (uint32_t)def_Color_Label_Input_Data,
                    (uint16_t)def_Pos_X_Label_Input_Data,
                    (uint16_t)def_Pos_Y_Label_Input_Data,
                    (uint16_t)strlen((char *)def_aStr_Label_Input_Data),
                    (uint8_t *)def_aStr_Label_Input_Data);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 3:
    // 入力 Data 初期表示 : 位置表示
    //--------------------------------------------------
    // 横長表示 右
    //--------------------------------------------------
    //--------------------------------------------------------------
    // LCD文字表示 : ASCII : Font選択 : ILI9341
    //--------------------------------------------------------------
    int16_Return = ILI9341_SPI_Display_ASCII(&STC_uint8_JobNum_LCD,
                                1,
                                (uint16_t)def_Font_ASCII_16x16,
                                (uint8_t)def_Dir_Disp_LCD_Sel, 1,
                                (uint32_t)def_Color_Input_Data,
                                (uint16_t)def_Pos_X_Data_Input,
                                (uint16_t)def_Pos_Y_Data_Input,
                                1, (uint8_t *)" ");
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    //------------------------------------------------------
    // 表示するSW番号をセット
    //------------------------------------------------------
    // Function 表示
    //------------------------------------------------------
    STC_int16_Num_SW_Start = (int16_t)def_Num_TP_SW_Func_BK;
    STC_int16_Num_SW_End = (int16_t)def_Num_TP_SW_Key_Func;
    //------------------------------------------------------


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 4:
    // Function 表示
    //--------------------------------------------------------------
    // タッチパネルのスイッチ表示 :
    //    開始スイッチ番号から終了スイッチ番号までを表示する。
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納された変数のポインタ

    // int16_t int16_Num_SW_Start : 開始スイッチ番号
    // int16_t int16_Num_SW_End : 終了スイッチ番号

    // uint8_t uint8_R : 反転表示フラグ
    //    0 : 通常表示
    //    1 : 反転表示

    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = Print_TP_SW_Num(&STC_uint8_JobNum_LCD,
                                   STC_int16_Num_SW_Start,
                                   STC_int16_Num_SW_End,
                                   (uint8_t)0);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    //------------------------------------------------------
    // 表示するSW番号をセット
    //------------------------------------------------------
    // 0 - 9, A - Z, Control Code 表示
    //------------------------------------------------------
    STC_int16_Num_SW_Start = (int16_t)def_Num_TP_SW_Key_0;
    STC_int16_Num_SW_End = (int16_t)def_Num_TP_SW_Key_LF;
    //------------------------------------------------------


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 5:
    // 0 - 9, A - Z, Control Code 表示
    //--------------------------------------------------------------
    // タッチパネルのスイッチ表示 :
    //    開始スイッチ番号から終了スイッチ番号までを表示する。
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納された変数のポインタ

    // int16_t int16_Num_SW_Start : 開始スイッチ番号
    // int16_t int16_Num_SW_End : 終了スイッチ番号

    // uint8_t uint8_R : 反転表示フラグ
    //    0 : 通常表示
    //    1 : 反転表示


    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = Print_TP_SW_Num(&STC_uint8_JobNum_LCD,
                                   STC_int16_Num_SW_Start,
                                   STC_int16_Num_SW_End,
                                   (uint8_t)0);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 6:
    //--------------------------------------------------------------
    // ASC/HEX表示
    //--------------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号が格納された変数のポインタ

    // int16_t int16_Sel : 表示選択
    //  0 : ASCII : ASC
    //  1 : HEX : HEX 1
    //  2 : HEX : HEX 2

    // 戻り値：
    //   -1 : 処理中
    //    0 : OK終了
    //    1 : NG終了
    //--------------------------------------------------------------
    int16_Return = Draw_Msg_ASC_HEX(&STC_uint8_JobNum_LCD,
                                    GLB_int16_Flag_AN_HEX);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }


    *puint8_JobNum = 0;  // 処理番号初期化
    // int16_Return : 結果
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}
